// DialogEditPool.cpp : implementation file
//

#include "stdafx.h"
#include "awana_grandprix_scoring.h"
#include "DialogEditPool.h"

#include "Model.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDialogEditPool dialog


CDialogEditPool::CDialogEditPool(CWnd* pParent /*=NULL*/)
	: CDialog(CDialogEditPool::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDialogEditPool)
	m_name = _T("");
	m_canRace = FALSE;
	m_id = _T("");
	//}}AFX_DATA_INIT

  mModel = 0;
  mHasAutosaved = false;
}


void CDialogEditPool::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDialogEditPool)
	DDX_Control(pDX, IDC_SPIN_NEWID, m_spin_newid);
	DDX_Control(pDX, IDC_LIST_POOL, m_list_pool);
	DDX_Text(pDX, IDC_EDIT_NAME, m_name);
	DDX_Check(pDX, IDC_CHECK_CANRACE, m_canRace);
	DDX_Text(pDX, IDC_EDIT_ID, m_id);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDialogEditPool, CDialog)
	//{{AFX_MSG_MAP(CDialogEditPool)
	ON_BN_CLICKED(IDC_BUTTON_LOAD, OnButtonLoad)
	ON_BN_CLICKED(IDC_BUTTON_SAVE, OnButtonSave)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE, OnButtonRemove)
	ON_BN_CLICKED(IDC_BUTTON_ADD, OnButtonAdd)
	ON_LBN_SELCHANGE(IDC_LIST_POOL, OnSelchangeListPool)
	ON_EN_CHANGE(IDC_EDIT_NAME, OnChangeEditName)
	ON_BN_CLICKED(IDC_CHECK_CANRACE, OnCheckCanrace)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDialogEditPool message handlers

void CDialogEditPool::refreshFromModel() {

  m_list_pool.ResetContent();
  
  CModel::tIntLIST theIDs;
  theIDs = mModel->getIDsInPool();

  for (CModel::tIntLIST::const_iterator theIter = theIDs.begin(); theIter != theIDs.end(); theIter++) {
    char theBuf[1024];

    int theID = *theIter;
    char* theName = mModel->getNameForID(theID);

    sprintf(theBuf, "%03d   %s", theID, theName);

    m_list_pool.AddString(theBuf);
    int theIndex = m_list_pool.FindString(0, theBuf);
    m_list_pool.SetItemData(theIndex, theID);
  }

}

void CDialogEditPool::OnButtonLoad() {
  CFileDialog theDialog(true);
  int theRet = theDialog.DoModal();

  if (theRet == IDOK) {
    FILE* theFile = fopen(theDialog.GetPathName(), "rt");
    mModel->load(theFile);
    fclose(theFile);
  }

  refreshFromModel();
}

void CDialogEditPool::OnButtonSave() {
  CFileDialog theDialog(false);
  int theRet = theDialog.DoModal();
	
  if (theRet == IDOK) {
    FILE* theFile = fopen(theDialog.GetPathName(), "w+t");
    mModel->save(theFile);
    fclose(theFile);
  }
}

void CDialogEditPool::OnButtonRemove() {
  if (!mHasAutosaved) {
    mModel->autosave();
    mHasAutosaved = true;
  }

  int theIndex = m_list_pool.GetCurSel();
  if (theIndex == LB_ERR) {
    return;
  }

  mModel->removeFromPool(m_list_pool.GetItemData(theIndex));
  m_list_pool.DeleteString(theIndex);
}

void CDialogEditPool::OnButtonAdd() {
  if (!mHasAutosaved) {
    mModel->autosave();
    mHasAutosaved = true;
  }

  UpdateData(TRUE);
  int theID = atoi(m_id);

  char theBuf[1024];
  sprintf(theBuf, "%03d", theID);

  if (!(mModel->inPool(theID))) {
    mModel->addToPool(theID);

    m_list_pool.AddString(theBuf);

    int theIndex = m_list_pool.FindString(0, theBuf);
    m_list_pool.SetItemData(theIndex, theID);
    m_list_pool.SetCurSel(theIndex);
  } else {
    m_list_pool.SelectString(0, theBuf);
  }
}

BOOL CDialogEditPool::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
  m_spin_newid.SetRange(0, mModel->getMaxCarID());
  m_spin_newid.SetPos(0);
  m_id.Format("%d", m_spin_newid.GetPos());

  refreshFromModel();

  UpdateData(FALSE);
  
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CDialogEditPool::OnSelchangeListPool() {
	int theIndex = m_list_pool.GetCurSel();
  if (theIndex == LB_ERR) {
    return;
  }

  int theID = m_list_pool.GetItemData(theIndex);
  char* theName = mModel->getNameForID(theID);
  bool theIsRaceable = mModel->isIDRaceable(theID);

  m_name = theName;
  m_canRace = theIsRaceable;
  
  UpdateData(FALSE);
}

void CDialogEditPool::OnChangeEditName() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.
	
  updateListFromUI();	
}

void CDialogEditPool::OnCheckCanrace() {
	updateListFromUI();
}

void CDialogEditPool::updateListFromUI() {
  int theIndex = m_list_pool.GetCurSel();
  if (theIndex == LB_ERR) {
    return;
  }

  if (!mHasAutosaved) {
    mModel->autosave();
    mHasAutosaved = true;
  }
  
  UpdateData(TRUE);

  int theID = m_list_pool.GetItemData(theIndex);

  // update model
  mModel->setNameForID(theID, m_name);
  if (m_canRace) {
    mModel->setIDRaceable(theID);
  } else {
    mModel->clearIDRaceable(theID);
  }

  // update list
  char theBuf[1024];

  sprintf(theBuf, "%03d   %s", theID, m_name);

  m_list_pool.DeleteString(theIndex);

  m_list_pool.AddString(theBuf);
  theIndex = m_list_pool.SelectString(0, theBuf);
  m_list_pool.SetItemData(theIndex, theID);
}
