// State.h: interface for the CState class.
// - data structure to store state for the program
// - if pool 2nd int is 0 then id is raceable
//
//////////////////////////////////////////////////////////////////////

#ifndef __state_h__
#define __state_h__

#ifdef _MSC_VER
  // microsoft vc++ specific
  #pragma warning(disable:4786)
#endif

#include <list>
#include <map>
#include <set>
#include <vector>

using namespace std;

#include "stdio.h"

class CName;

class CState {
  public:
    struct sIDScore {
      int mID;
      int mScore;

      sIDScore() {
        mID = -1;
        mScore = -1;
      }
      
      sIDScore(int pID, int pScore) {
        mID = pID;
        mScore = pScore;
      }
    };

    typedef list<int> tIntLIST;

    struct sIntScoreList {
      int mOrder;
      tIntLIST mList;

      sIntScoreList() {
        mOrder = 0;
      }
    };

    typedef map<int, int> tIntIntMAP;
    typedef map<int, sIDScore> tIDScoreMAP;
    typedef map<int, sIntScoreList> tIntIntScoreListMAP;

    tIntIntMAP mPool;
    tIntIntScoreListMAP mScore;
    tIntIntMAP mSelection[2];
    tIDScoreMAP mTrackScore[2];

	  CState();
	  virtual ~CState();

    // undo
    // - returns pointer to 'last' CState
    CState* undo();
    
    // can we undo?
    bool canUndo();

    // redo
    // - returns pointer to 'next' CState
    CState* redo();

    // can we redo?
    bool canRedo();

    // get first state
    CState* firstState();

    // get last state
    CState* lastState();

    // clone
    // - clones current state, and links it into the 'next' state.
    // - if next state already exists, clean up all memory associated with it
    // - returns pointer to next state
    CState* clone();

    // initialize pool with list of names
    void initPoolWithCName(const CName& pNames);

    // is the state clear? (no data loaded)
    bool isClear();

    // load
    // - load a set of CStates from file
    // - starts the load process from the current CState and overwrites the 'next' pointer
    void load(FILE* pFile);

    // save
    // - saves this CState and everything previous to it
    void save(FILE* pFile);

  protected:

    // cleans up everything in the 'next' pointer and beyond
    // - sets 'next' pointer to null
    void cleanupNext();

    // clear current state
    // - doesn't clear the 'last' pointer
    void clearState();

    // creates a new state attached to the 'next' pointer
    void newState();

    // last and next states
    CState* mLast;
    CState* mNext;

};

#endif // __state_h__
